# Generated by Django 6.0.2 on 2026-03-11 15:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("legacy_data", "0002_alter_legacycalculationid_options_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="LegacyLog",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("group", models.TextField(blank=True, null=True)),
                ("logfile", models.TextField(blank=True, null=True)),
                ("input_validation", models.TextField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Generic Log",
                "verbose_name_plural": "Generic Logs",
                "db_table": "generic_app_log",
                "managed": False,
            },
        ),
    ]
